// dependencies
const { parseURL } = require('ioredis/built/utils');
const { Worker, Queue, QueueEvents } = require('bullmq');

// local
const { takeSnapshot } = require('./utils');

const redisConnection = parseURL('redis://localhost:6379/1');

const queue = new Queue('oom-test', { connection: redisConnection });
const queueEvents = new QueueEvents('oom-test', { connection: redisConnection });

const sleep = (ms) => {
  return new Promise(resolve => setTimeout(resolve, ms));
}

const testJob = async (index) => {
  console.log(`Running testJob (${index})`);

  let job = await queue.add('one');
  const result = await job.waitUntilFinished(queueEvents);

  try {
    await job.remove();
    await queue.clean(0);
  } catch (e) {
    console.log(`Got error ${e}`);
  }

  return result;
};

let jobOneCount = 0;
const jobOne = async (job) => {
  const id = `[${job.id}]`;
  console.log(`jobOne id = (${id}) and jobOneCount = (${jobOneCount})`);
  
  if (jobOneCount % 25 === 0) {
    takeSnapshot('worker-leak');
  }
  jobOneCount++;

  await sleep(50);
  return `${id}-${'a'.repeat(1024 * 1024)}`;
}

const jobAll = async (job) => {
  const id = `[${job.id}]`;
  console.log(`jobAll id = (${id})`);

  for (let i = 0; i < 105; i += 1) {
    await testJob(i);
  }

  return 0;
}

const processor = (job) => {
  if (job.name === 'one') {
    return jobOne(job);
  }

  return jobAll(job);
}

const worker = new Worker('oom-test', processor, { connection: redisConnection, concurrency: 3 });

worker.on('error', (err) => {
  console.log(`Error from worker ${worker.name}:`, { context: err });
});
